/*
 * Copyright (c) 2007-2008
 *     IS/AIST-ST2I/CNRS Joint Japanese-French Robotics Laboratory (JRL).
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_VIAMTYPES
#define H_VIAMTYPES

#define VIAM_OPENCV_SUPPORT
#define VIAM_DC1394_SUPPORT


/* --- Generic types ------------------------------------------------- */

typedef enum viam_onoff_t {
  VIAM_OFF =		0,
  VIAM_ON =		1
} viam_onoff_t;

typedef enum viam_errorcode_t {
  Eviamlib_OK = 0,
  Eviamlib_MEMORY,
  Eviamlib_OPENCV,
  Eviamlib_FWBUS,
  Eviamlib_FILE_IO,
  Eviamlib_NO_CAMERA,
  Eviamlib_NOT_IMPLEMENTED,
  Eviamlib_DUPLICATE_ID,
  Eviamlib_BAD_ID,
  Eviamlib_NO_SUCH_DEVICE,
  Eviamlib_NO_SUCH_IMAGE,
  Eviamlib_NO_SUCH_CAMERA,
  Eviamlib_NO_SUCH_BANK,
  Eviamlib_NO_SUCH_FILTER,
  Eviamlib_NOT_CONFIGURED,
  Eviamlib_CONFIGURED,
  Eviamlib_UNSUPPORTED_HWMODE,
  Eviamlib_INVALID_VALUE,
  Eviamlib_IMAGE_GEOMETRY,
  Eviamlib_BAD_IMAGE_COUNT,
  Eviamlib_CALIBRATING,
  Eviamlib_NOT_CALIBRATED
} viam_errorcode_t;


/* --- Hardware output format ---------------------------------------- */

typedef enum viam_hwcrop_t {
  VIAM_HW_FIXED,
  VIAM_HW_CROP
} viam_hwcrop_t;

typedef enum viam_hwsize_t {
  VIAM_HWSZ_INVALID,
  VIAM_HWSZ_160x120,
  VIAM_HWSZ_320x240,
  VIAM_HWSZ_512x384,
  VIAM_HWSZ_640x480,
  VIAM_HWSZ_800x600,
  VIAM_HWSZ_1024x768,
  VIAM_HWSZ_1280x960,
  VIAM_HWSZ_1600x1200
} viam_hwsize_t;
#define viam_hwsize_vals	(VIAM_HWSZ_1600x1200 +1)

typedef enum viam_hwformat_t {
  VIAM_HWFMT_INVALID,
  VIAM_HWFMT_MONO8,
  VIAM_HWFMT_MONO16,
  VIAM_HWFMT_YUV411,
  VIAM_HWFMT_YUV422,
  VIAM_HWFMT_YUV444,
  VIAM_HWFMT_RGB888
} viam_hwformat_t;
#define viam_hwformat_vals	(VIAM_HWFMT_RGB888 +1)

typedef enum viam_hwfps_t {
  VIAM_HWFPS_INVALID,
  VIAM_HWFPS_1_875,
  VIAM_HWFPS_3_75,
  VIAM_HWFPS_7_5,
  VIAM_HWFPS_15,
  VIAM_HWFPS_30,
  VIAM_HWFPS_60,
  VIAM_HWFPS_120,
  VIAM_HWFPS_240,
  VIAM_HWFPS_TRIGGER_MODE0_LOW,
  VIAM_HWFPS_TRIGGER_MODE0_HIGH,
  VIAM_HWFPS_TRIGGER_MODE1_LOW,
  VIAM_HWFPS_TRIGGER_MODE1_HIGH
} viam_hwfps_t;
#define viam_hwfps_extern	(VIAM_HWFPS_TRIGGER_MODE0_LOW)
#define viam_hwfps_vals		(VIAM_HWFPS_TRIGGER_MODE1_HIGH +1)

typedef struct viam_hwmode_t {
  viam_hwsize_t size;
  viam_hwformat_t format;
  viam_hwcrop_t crop;
  viam_hwfps_t fps;
} viam_hwmode_t;


/* --- Image filters ------------------------------------------------- */

typedef enum viam_filtertype_t {
  VIAM_FILTER_NONE,
  VIAM_FILTER_FORMAT,
  VIAM_FILTER_SUBSAMPLE,
  VIAM_FILTER_SCALE,
#ifdef VIAM_OPENCV_SUPPORT
  VIAM_FILTER_DISTORT,
  VIAM_FILTER_RECTIFY,
#endif /* VIAM_OPENCV_SUPPORT */
  VIAM_FILTER_BRIGHTNESS,
  VIAM_FILTER_EXPOSURE,
  VIAM_FILTER_SHARPNESS,
  VIAM_FILTER_GAIN,
  VIAM_FILTER_SHUTTER,
  VIAM_FILTER_WHITE_BALANCE,
  VIAM_FILTER_HUE,
  VIAM_FILTER_SATURATION,
  VIAM_FILTER_GAMMA,
  VIAM_FILTER_TEMPERATURE

#ifdef VIAM_OPENCV_SUPPORT
  ,
  VIAM_FILTER_LOG
#endif /* VIAM_OPENCV_SUPPORT */
} viam_filtertype_t;

/** Parameters of VIAM_FILTER_FORMAT filter */
typedef enum viam_filterformat_t {
  VIAM_FILTER_DEMUX_EVENPX,
  VIAM_FILTER_DEMUX_ODDPX,
#ifdef VIAM_OPENCV_SUPPORT
  VIAM_FILTER_BAYER_BGGR,
  VIAM_FILTER_BAYER_GRBG,
  VIAM_FILTER_BAYER_RGGB,
  VIAM_FILTER_BAYER_GBRG,
  VIAM_FILTER_RGB_TO_BGR,
  VIAM_FILTER_RGB_TO_GRAY,
  VIAM_FILTER_BGR_TO_GRAY,
#endif /* VIAM_OPENCV_SUPPORT */
  VIAM_FILTER_YUV422_TO_BGR,
  VIAM_FILTER_YUV411_TO_BGR,
  VIAM_FILTER_YUV444_TO_BGR
} viam_filterformat_t;

typedef enum viam_filtermethod_t {
  VIAM_FILTER_INVALID =			0,
  VIAM_FILTER_OFF =			0x1,
  VIAM_FILTER_HARDWARE =		0x2,
  VIAM_FILTER_SOFTWARE =		0x4
} viam_filtermethod_t;

typedef enum viam_filterautomode_t {
  VIAM_FILTER_AUTOINVALID =		0,
  VIAM_FILTER_MANUAL =			0x1,
  VIAM_FILTER_HARDWARE_AUTO =		0x2,
  VIAM_FILTER_HARDWARE_ONE_PUSH =	0x4,
  VIAM_FILTER_SOFTWARE_AUTO =		0x8,
  VIAM_FILTER_SOFTWARE_ONE_PUSH =	0x10
} viam_filterautomode_t;

typedef struct viam_geometry_t {
  unsigned int height, width;
  float scalex, scaley;
  float offx, offy;
  unsigned int depth;
  unsigned int channels;
} viam_geometry_t;


/* --- Image calibration --------------------------------------------- */

typedef enum viam_caltype_t {
  VIAM_CAL_MONO,
  VIAM_CAL_STEREO
} viam_caltype_t;

typedef struct viam_cameracalibration_t {
  double intrinsic[9];	/**< intrinsic matrix (homogeneous 3x3) */
  double intrirect[9];	/**< intrinsic matrix of rectified camera (homogeneous 3x3) */
  double distortion[5];	/**< distortion coeffs (1x5: r1, r2, t1, t2, r3) */
  double rectification[9];/**< rectification matrix (homogeneous 3x3) */
  double rotation[9];	/**< 3D rotation matrix from rectified to original (3x3) */

  int width;		/**< width of the calibration images */
  int height;		/**< height of the calibration images */
} viam_cameracalibration_t;

typedef struct viam_bankcalibration_t {
  viam_caltype_t type;	/**< MONO or STEREO */
  double baseline;	/**< baseline in meters */
  double pbaseline;	/**< baseline in pixels */
} viam_bankcalibration_t;


#endif /* H_VIAMTYPES */
